//********************************************************
// uWatch
// RPN Calculator Functions
// Version 1.3
// Last Update: 12th June 08
// Copyright(c) 2008 David L. Jones
// Written for the Microchip C30 Compiler
// Target Device: PIC24FJ64GA004 (44pin)
// http://www.calcwatch.com
// EMAIL: david@alternatezone.com

// NOTE: This code is designed to be inserted inline into the uWatch-Main.C function
//       Most variables are passed as globals and are defined in uWatch-Main.C
//********************************************************

/*********************************************************
This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**********************************************************/


//***********************************
// pushes the stack up and leaves the Xreg intact
void PushStackUp(void)
{
	Treg=Zreg;
	Zreg=Yreg;
	Yreg=Xreg;
	UpdateYregDisplay();		//Yreg has changed, so keep display register up to date
//	strcpy(DisplayXreg,"");		//clear the display Xreg
}

//**********************************
// Drops the stack down and leaves Treg intact making it useful as a "constant" register
// Xreg is handled in the calling routine
void DropStack(void)
{
	Yreg=Zreg;
	Zreg=Treg;
}


//**********************************
// Performs the function passed by the MenuItem number
// MenuItem numbers onm screen are as follows:
// " 1  2  3  4 "
// " 5  6  7  8 "
// CurrentMenu holds the desired menu display number
void ProcessMenuItemRPN(int MenuItem)
{	
	double temp;
		
	switch(CurrentMenu)
		{
		case 0:			//MENU #1
			{
			Menu0(MenuItem);
			break;
			}
		
		case 1:
			{
			Menu1(MenuItem);
			break;
			}
		
		case 2:
			{
			switch(MenuItem)
				{
				case 1:		//PI key
					{
					if (ValueEntered==TRUE) PushStackUp();
					strcpy(DisplayXreg,PIstring);
					ValueEntered=FALSE;
					SwitchMenuOff();					
					break;
					}
				case 2:		//X^Y key
					{
					CompleteXreg();		//enter value on stack if needed
					Xreg=pow(Yreg,Xreg); 
					DropStack();
					UpdateDisplayRegs();	//update display again
					SwitchMenuOff();					
					break;
					}
				case 3:		//e^X key
					{
					CompleteXreg();		//enter value on stack if needed
					Xreg=exp(Xreg);				
					UpdateDisplayRegs();	//update display again
					SwitchMenuOff();
					break;
					}
				case 4:		// R-P key
					{
					CompleteXreg();			//enter value on stack if needed
					RtoP();
					UpdateDisplayRegs();	//update display again
					SwitchMenuOff();
					break;
					}
				case 5:		// P-R key
					{
					CompleteXreg();		//enter value on stack if needed					
					PtoR();
					UpdateDisplayRegs();	//update display again
					SwitchMenuOff();
					break;
					}
				case 6:		//Parallel key 
					{
					CompleteXreg();		//enter value on stack if needed
					Xreg=(Yreg*Xreg)/(Yreg+Xreg);
					DropStack();
					UpdateDisplayRegs();	//update display again
					SwitchMenuOff();
					break;
					}
				}
			break;
			}

		case 3:
			{
			switch(MenuItem)
				{
				case 1:			//Keystroke RECORD selected
					{
					CompleteXreg();		//enter value on stack if needed
					KeyRecord();
					SwitchMenuOff();
					break;
					}
				case 2:			//keystroke PLAYBACK
					{
					CompleteXreg();		//enter value on stack if needed
					KeyReplay();
					SwitchMenuOff();
					break;
					}
				case 3:			// conversions
					{
					Conversions();
					break;
					}
				case 4:			//X! key 
					{
					CompleteXreg();		//enter value on stack if needed
					Xreg=Factorial(Xreg);
					UpdateDisplayRegs();	//update display again
					SwitchMenuOff();
					break;
					}
				case 5:			//cube root key
					{
					CompleteXreg();		//enter value on stack if needed
					Xreg=pow(Xreg,(1.0/3.0));	
					UpdateDisplayRegs();	//update display again
					SwitchMenuOff();
					break;
					}
				case 6:			// HEX entry key
					{
					HexEntry();
					break;
					}
				}
			break;
			}
		}
}

//**********************************
void ProcessNumberKey(char digit[])
{
	if ((strlen(DisplayXreg)<MaxLCDdigits)&&(MenuMode==FALSE))
		{
		if (ValueEntered==TRUE) 	//if this is the first digit pressed
			{
			if (EnableXregOverwrite==FALSE)	//check to see if we don't have to overwrite the Xreg
				PushStackUp();				//push the stck up for the first key entry, i.e. *don't* overwrite the Xreg
			strcpy(DisplayXreg,"");		//clear (overwrite)what was in the Xreg
			EnableXregOverwrite=FALSE;	//disable overwriting the Xreg for future key presses
			}
		strcat(DisplayXreg,digit);
		ValueEntered=FALSE;
		MenuMode=FALSE;
		UpdateLCDline1(DisplayYreg);
		UpdateLCDline2(DisplayXreg);
		}
}


//***********************************
// The main RPN calculator routine
// Note that all variables are global
void RPNcalculator(void)
{
	unsigned int Key;			//keypress variables
	double TEMPreg;				//temp register for calculations
	strcpy(DisplayXreg,"");		//clear the display register to start
	strcpy(DisplayYreg,"");
	DecimalIncluded=FALSE;
	MinusIncluded=FALSE;	
	MinusIncludedInExponent=FALSE;
	ExponentIncluded=FALSE;
	CurrentMenu=0;
	ValueEntered=TRUE;
	UpdateXregDisplay();
	UpdateYregDisplay();
	UpdateLCDline1(DisplayYreg);
	UpdateLCDline2(DisplayXreg);

	while(TRUE) 	//main keypress loop is infinite.
		{
		do {			//wait for a keypress
			Key=KeyScan();
		} while(Key==0);
		
		ResetSleepTimer();		
		//start to process the keypress

		if (Key==KeyMode)		// mode key was pressed, exit calc mode
			{
			DelayMs(KeyDelay);
			return;
			}

		NextMode=FALSE;				//user pressed some key other than MODE, so ensure that when exit we go back to the time/date display
		ResetSleepTimer();

		if (Key==Key1)				//user has pressed the 1 key
			{
			if (MenuMode==FALSE) ProcessNumberKey("1");
			//key is ignored when menu is on.
			}

		if (Key==Key2)				//user has pressed the 2 key
			{
			if (MenuMode==FALSE) ProcessNumberKey("2");
			//key is ignored when menu is on.
			}

		if (Key==Key3)				//user has pressed the 3 key
			{
			if (MenuMode==FALSE) ProcessNumberKey("3");
			//key is ignored when menu is on.
			}

		if (Key==Key4)				//user has pressed the 4 key
			{
			if (MenuMode==FALSE) ProcessNumberKey("4");
			else ProcessMenuItemRPN(4);	//user is in menu mode, so process as a menu key
			}

		if (Key==Key5)				//user has pressed the 5 key
			{
			if (MenuMode==FALSE) ProcessNumberKey("5");
			else ProcessMenuItemRPN(5);	//user is in menu mode, so process as a menu key
			}

		if (Key==Key6)				//user has pressed the 6 key
			{
			if (MenuMode==FALSE) ProcessNumberKey("6");
			else ProcessMenuItemRPN(6);	//user is in menu mode, so process as a menu key
			}

		if (Key==Key7)				//user has pressed the 7 key
			{
			if (MenuMode==FALSE) ProcessNumberKey("7");
			else ProcessMenuItemRPN(1);	//user is in menu mode, so process as a menu key
			}
			
		if (Key==Key8)				//user has pressed the 8 key
			{
			if (MenuMode==FALSE) ProcessNumberKey("8");
			else ProcessMenuItemRPN(2);	//user is in menu mode, so process as a menu key
			}

		if (Key==Key9)				//user has pressed the 9 key
			{
			if (MenuMode==FALSE) ProcessNumberKey("9");
			else ProcessMenuItemRPN(3);	//user is in menu mode, so process as a menu key
			}
			
		if (Key==Key0)				//user has pressed the 0 key
			{
			if (MenuMode==FALSE) ProcessNumberKey("0");
			//key is ignored when menu is on.
			}
			
		if (Key==KeyPoint)			//user has pressed the DECIMAL POINT key
			{
			if (strlen(DisplayXreg)<MaxLCDdigits)
				//only do if decimal point does not already exist AND there is no exponent
				if ((DecimalIncluded==FALSE) && ExponentIncluded==FALSE)		
					{
					if (ValueEntered==TRUE) 	//user has pressed POINT as the first digit
						{
						strcpy(DisplayXreg,"0.");	//decimal point needs a 0 added to the start
						}
					else		
						strcat(DisplayXreg,".");
					DecimalIncluded=TRUE;
					ValueEntered=FALSE;
					UpdateLCDline1(DisplayYreg);
					UpdateLCDline2(DisplayXreg);
					}
			}
		if (Key==KeyEnter)			//user has pressed the ENTER key
			{
			if (MenuMode==FALSE)	//this key is not used in MENU mode
				{
				CompleteXreg();
				PushStackUp();
				ResetFlags();
				EnableXregOverwrite=TRUE;	//overwite the flag and force the Xreg to be overwritten on the next entry
				UpdateDisplayRegs();
				}
			}
		if (Key==KeyPlus)			//user has pressed the PLUS key
			{
			if (MenuMode==FALSE)	//this key is not used in MENU mode
				{
				CompleteXreg();
				ResetFlags();
				Xreg=Xreg+Yreg;		//perform PLUS operation
				DropStack();
				UpdateDisplayRegs();
				}
			}
		if (Key==KeyMinus)			//user has pressed the MINUS key
			{
			if (MenuMode==FALSE)		//this key is not used in MENU mode
				{
				CompleteXreg();
				ResetFlags();
				Xreg=Yreg-Xreg;			//perform MINUS operation
				DropStack();
				UpdateDisplayRegs();
				}
			}
		if (Key==KeyMult)				//user has pressed the MULTIPLY key
			{
			if (MenuMode==FALSE)		//this key is not used in MENU mode
				{
				CompleteXreg();
				ResetFlags();
				Xreg=Xreg*Yreg;			//perform MULTIPLY operation
				DropStack();
				UpdateDisplayRegs();
				}
			}
		if (Key==KeyDiv)				//user has pressed the DIVIDE key
			{
			if (MenuMode==FALSE)			//this key is not used in MENU mode
				{
				CompleteXreg();
				ResetFlags();
				Xreg=Yreg/Xreg;		//perform DIVIDE operation
				DropStack();
				UpdateDisplayRegs();
				}
			}
		if (Key==KeyClear)			//user has pressed the CLEAR key
			{
			InverseKey=FALSE;		//reset the function flags
			HYPkey=FALSE;

			if (MenuMode==TRUE)		//menu is being displayed
				{
				MenuMode=FALSE;		//switch off menu mode
				UpdateLCDline1(DisplayYreg);	//display the X & Y regs again
				UpdateLCDline2(DisplayXreg);
				}
			else
				{
				if (ValueEntered==FALSE)		//only clear if there is something in the display register to clear
					{
					strcpy(DisplayXreg,"");		//ONLY clear the display register
					UpdateLCDline1(DisplayYreg);
					UpdateLCDline2(DisplayXreg);
					ResetFlags();
					}
				else						//clear the X register instead and DROP the stack
					{
					Xreg=Yreg;				//drop X reg from the stack
					DropStack();
					Treg=0;
					UpdateDisplayRegs();
					ResetFlags();
					}
				}
			}

		if (Key==KeyMenu)
			{
			CalcMenu();
			}

		if (Key==KeyEXP)
			if (MenuMode==FALSE)			//this key is not used in MENU mode			
				if (ExponentIncluded==FALSE)	//can't add exponent twice
					{
					ExponentIncluded=TRUE;
					if (ValueEntered==TRUE) strcpy(DisplayXreg,"1e");	// EXP is first key pressed, so add a 1 to the front
					else strcat(DisplayXreg,"e");		
					ValueEntered=FALSE;	
					UpdateLCDline2(DisplayXreg);		//update the display
					}

		if (Key==KeyXY)
			if (MenuMode==FALSE)			//this key is not used in MENU mode			
				{
				CompleteXreg();		//enter value on stack if needed
				TEMPreg=Xreg;			
				Xreg=Yreg;			//swap and X and Y regs
				Yreg=TEMPreg;
				UpdateDisplayRegs();	//update display again
				}

		if (Key==KeyLP)
			if (MenuMode==FALSE)			//this key is not used in MENU mode			
				{
				CompleteXreg();		//enter value on stack if needed
				TEMPreg=Xreg;			
				Xreg=Yreg;			// roll the stack DOWN
				Yreg=Zreg;
				Zreg=Treg;
				Treg=TEMPreg;
				UpdateDisplayRegs();	//update display again
				}

		if (Key==KeyRCL)
			{
						//recalls the one of the user defined memory registers into the Xreg
			if (MenuMode==FALSE)			//this key is not used in MENU mode			
				{
				CompleteXreg();
				StoreRecall();
				UpdateDisplayRegs();	//update display again
				}
			}

		if (Key==KeySign)
						//changes the sign of the mantissa or exponent
			{
			SignKey();
			}

		}	//end of entire loop
}

